Package: bamsignals
Type: Package
Title: Extract read count signals from bam files
Version: 1.42.0
Date: 2025-06-21
Authors@R: c( person(given="Alessandro",family="Mammana",
        email="mammana@molgen.mpg.de", role=c("aut", "cre")),
        person(given="Johannes",family="Helmuth",
        email="johannes.helmuth@laborberlin.com", role=c("aut")))
Author: Alessandro Mammana [aut, cre], Johannes Helmuth [aut]
Maintainer: Johannes Helmuth <johannes.helmuth@laborberlin.com>
Description: This package allows to efficiently obtain count vectors
    from indexed bam files. It counts the number of reads in given
    genomic ranges and it computes reads profiles and coverage
    profiles. It also handles paired-end data.
License: GPL-2
Depends: R (>= 3.5.0)
Imports: methods, BiocGenerics, Rcpp (>= 0.10.6), IRanges,
        GenomicRanges
Suggests: testthat (>= 0.9), Rsamtools, BiocStyle, knitr, rmarkdown
LinkingTo: Rcpp, Rhtslib (>= 1.13.1)
SystemRequirements: GNU make
biocViews: DataImport, Sequencing, Coverage, Alignment
VignetteBuilder: knitr
NeedsCompilation: yes
RoxygenNote: 5.0.1
URL: https://github.com/lamortenera/bamsignals
BugReports: https://github.com/lamortenera/bamsignals/issues
git_url: https://git.bioconductor.org/packages/bamsignals
git_branch: RELEASE_3_22
git_last_commit: 47952a1
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
Packaged: 2025-10-30 02:36:21 UTC; biocbuild
