# Generated by roxygen2: do not edit by hand

S3method(as.list,CountSignals)
export(alignSignals)
export(bamCount)
export(bamCoverage)
export(bamProfile)
exportClasses(CountSignals)
exportMethods("[")
exportMethods(alignSignals)
exportMethods(as.list)
exportMethods(bamCount)
exportMethods(bamCoverage)
exportMethods(bamProfile)
exportMethods(length)
exportMethods(width)
import(BiocGenerics)
import(GenomicRanges)
import(IRanges)
import(Rcpp)
import(methods)
useDynLib(bamsignals, .registration=TRUE)
