# This file was generated by Rcpp::compileAttributes
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

checkList <- function(l, ss) {
    .Call('bamsignals_checkList', PACKAGE = 'bamsignals', l, ss)
}

fastWidth <- function(l, ss) {
    .Call('bamsignals_fastWidth', PACKAGE = 'bamsignals', l, ss)
}

pileup_core <- function(bampath, gr, tlen_filter, mapqual = 0L, binsize = 1L, shift = 0L, ss = FALSE, requiredF = 0L, filteredF = -1L, pe_mid = FALSE, maxgap = 16385L) {
    .Call('bamsignals_pileup_core', PACKAGE = 'bamsignals', bampath, gr, tlen_filter, mapqual, binsize, shift, ss, requiredF, filteredF, pe_mid, maxgap)
}

coverage_core <- function(bampath, gr, tlen_filter, mapqual = 0L, requiredF = 0L, filteredF = -1L, tspan = FALSE, maxgap = 16385L) {
    .Call('bamsignals_coverage_core', PACKAGE = 'bamsignals', bampath, gr, tlen_filter, mapqual, requiredF, filteredF, tspan, maxgap)
}

writeSamAsBamAndIndex <- function(sampath, bampath) {
    .Call('bamsignals_writeSamAsBamAndIndex', PACKAGE = 'bamsignals', sampath, bampath)
}

